/*  KRunning - DigitalClock.cpp
 *  Copyright (C) 2002 by Andreas Scherf
 *  Begin Mon Mai 6 2002 write to scherfa@web.de if you found a bug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "DigitalClock.h"
#include <qdatetime.h>

// DEBUG
#include <iostream>
#include <qvgroupbox.h>
//
// Constructs a DigitalClock widget with a parent and a name.
//

DigitalClock::DigitalClock (QWidget * parent, const char *name) : QWidget (parent, name)
{

    QVBoxLayout    *vboxLayout = new QVBoxLayout (this, 0, 4);
    QHBoxLayout     *hboxLayout = new QHBoxLayout (0, 0, 6, "hboxLayout");

    label = new QLabel (this);
    //label->setText();
    vboxLayout->addWidget (label);

    lcdNumber = new QLCDNumber (this);
    lcdNumber->setSegmentStyle (QLCDNumber::Filled);
    lcdNumber->setNumDigits (8);

    lcdNumber->setLineWidth (2);	// set frame line width
    vboxLayout->addWidget (lcdNumber);

    sepLine = new QFrame (this);
    sepLine->setFrameStyle (QFrame::HLine | QFrame::Sunken);
    vboxLayout->addWidget (sepLine);

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    hboxLayout->addItem( spacer );
    message = new QLabel (this);

    message->setPaletteForegroundColor(QColor(255,0,0));
    /*v*/
    hboxLayout->addWidget (message);

    hboxLayout->addItem( spacer );
    vboxLayout->addLayout(hboxLayout);


    showStartTimer = -1;		// not showing starttime

    runnumber = -1;

    showStartTimeOnly = false;
    showCurrentTime = false;
    timestring=QString("hh:mm:ss");
}


DigitalClock::~DigitalClock ()
{
    delete          label;
    delete          lcdNumber;
    delete          sepLine;
    delete          message;
}

void DigitalClock::start () {
    showTime ();			// display the current time
    if (timestring=="hh:mm:ss.zzz") {
        lcdNumber->setNumDigits (12);
        killTimer(normalTimer);
        normalTimer=startTimer(1);
    }else {
        lcdNumber->setNumDigits (8);
        killTimer(normalTimer);
        normalTimer = startTimer (500);
    }	// 1/2 second timer events        500
}



//
// Handles timer events for the digital clock widget.
// There are two different timers; one timer for updating the clock
// and another one for switching back from date mode to time mode.
//

void DigitalClock::timerEvent (QTimerEvent * e) {
    if (e->timerId () == showStartTimer)	// stop showing date
        stopStartTime ();
    else if (showStartTimer == -1 && !showStartTimeOnly) {	// not showing date
        showTime ();
    } else {
        showStartTime ();
    }
}

//
// Enters date mode when the left mouse button is pressed.
//

void DigitalClock::mousePressEvent (QMouseEvent * e) {
    if (e->button () == QMouseEvent::LeftButton)	// left button pressed
        showStartTime ();
}




/** getTimeDiff - Main Function which calculates the Time Difference from Start to Now time.
 ** @author Andreas Scherf
 ** @version 0.3
 **/
QTime DigitalClock::getTimeDiff (const QTime & t) {
    QTime           time = QTime ();

    if (timestring=="hh:mm:ss.zzz")  {
        time = time.addMSecs ((time.msecsTo (t)) - (time.msecsTo (starttime)));
    } else {
        time = time.addSecs ((time.secsTo (t)) - (time.secsTo (starttime)));
    }
    if (time.isValid ()) {
        return time;
    } else {
        return QTime::currentTime ();
    }
}


//
// Shows the current date in the internal lcd widget.
// Fires a timer to stop showing the date.
//

void DigitalClock::showStartTime () {
    if (showStartTimer != -1)	// already showing date
        return;

    QString s=starttime.toString(timestring);

    lcdNumber->display (s);	// sets the LCD number/text
    showStartTimer = startTimer (2000);	// keep this state for 2 secs
}

//
// Stops showing the date.
//

void DigitalClock::stopStartTime ()
{
    killTimer (showStartTimer);
    showStartTimer = -1;
    showTime ();
}

//
// Shows the current time in the internal lcd widget.
//

void DigitalClock::showTime () {
    QString  s;
    QTime time;
    if (!showCurrentTime) {
        time = getTimeDiff (QTime::currentTime ());//.left(5);
    } else {
        time = QTime::currentTime();
    }

    s = time.toString (timestring);


    lcdNumber->display (s);	// set LCD number/text
}

QString DigitalClock::getCurrentRunningTime () {
    QString  result;
    result = getTimeDiff (QTime::currentTime ()).toString (timestring);	//.left(5);
    return result;
}


void DigitalClock::setTimeFormat(const QString &ts) {
    timestring=ts;
}

void DigitalClock::setShowCTime (bool flag) {
    showCurrentTime=flag;
}

void DigitalClock::setStartTime (const QString & time) {
    starttime = QTime::fromString (time, Qt::TextDate);
}

void DigitalClock::setMessageText (const QString & text) {
    message->setText (text);
}

void DigitalClock::setMessageText (const int &number) {
    message->setNum (number);
}

void DigitalClock::setMessageFont (const QFont & font) {
    message->setFont( font );
}

void DigitalClock::setLabelText (const QString & text) {
    label->setText (text);
}

void DigitalClock::setDisplayTime(const QString &dtime) {
    lcdNumber->display(dtime);
}
